/**
 * The Iterable module provides tools for working with Typescript's Iterable<T> type in a functional way.
 *
 * In functional jargon, this module provides a monadic interface over Typescript's Iterable<T>.
 *
 * This nodule does not implements fuctions that would lead to degrading perfomance of Iterable. Any Iterable can be easly converted to Array
 *
 * @since 1.0.0
 */
import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, } from "fp-ts/lib/Apply";
import * as chainable from "fp-ts/lib/Chain";
import { bindTo as bindTo_, flap as flap_, let as let_, } from "fp-ts/lib/Functor";
import * as O from "fp-ts/lib/Option";
import { identity, pipe } from "fp-ts/lib/function";
import { asUnit as asUnit_, as as as_, dual, tapIO as tapIO_, tap as tap_, } from "./internal";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export const URI = "Iterable";
/**
 * Return a `Iterable` with elements initialized with `f(i)`.
 * Iterable stops when f return O.none
 *
 * @category constructors
 * @since 1.0.0
 */
export const makeByWithIndex = (f) => ({
    *[Symbol.iterator]() {
        for (let i = 0, el = f(i); O.isSome(el); i++, el = f(i)) {
            yield el.value;
        }
    },
});
/**
 * @category constructors
 * @since 1.0.0
 */
export const makeBy = (f) => makeByWithIndex(() => f());
/**
 * @category constructors
 * @since 1.0.0
 */
export const unfold = (b, f) => {
    let bb = b;
    return {
        *[Symbol.iterator]() {
            for (let mt = f(bb); O.isSome(mt); mt = f(bb)) {
                const [a, b] = mt.value;
                bb = b;
                yield a;
            }
        },
    };
};
/**
 * @category conversions
 * @since 1.0.0
 */
export const fromIO = (ma) => ({
    *[Symbol.iterator]() {
        yield ma();
    },
});
/**
 * @category conversions
 * @since 1.0.0
 * isn't it weird? all arrays are iterables
 */
export const fromReadonlyArray = (ax) => ({
    *[Symbol.iterator]() {
        for (const a of ax) {
            yield a;
        }
    },
});
/**
 * @category constructors
 * @since 1.0.0
 */
export const of = (a) => ({
    *[Symbol.iterator]() {
        yield a;
    },
});
/**
 * @category instances
 * @since 1.0.0
 */
export const Pointed = { URI, of };
/**
 * @category mapping
 * @since 1.0.0
 */
export const mapWithIndex = (f) => (fa) => {
    let i = 0;
    return {
        *[Symbol.iterator]() {
            for (const a of fa) {
                yield f(i++, a);
            }
        },
    };
};
/**
 * @category mapping
 * @since 1.0.0
 */
export const map = (f) => mapWithIndex((_i, a) => f(a));
/**
 * @category instances
 * @since 1.0.0
 */
export const Functor = {
    URI,
    map: (fa, f) => map(f)(fa),
};
/**
 * @category instances
 * @since 1.0.0
 */
export const FunctorWithIndex = {
    ...Functor,
    mapWithIndex: (fa, f) => mapWithIndex(f)(fa),
};
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const as = dual(2, as_(Functor));
/**
 * Maps the value to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export const asUnit = asUnit_(Functor);
/**
 * @category mapping
 * @since 1.0.0
 */
export const flap = flap_(Functor);
/**
 * @category apply
 * @since 1.0.0
 */
export const ap = (fa) => (fab) => ({
    *[Symbol.iterator]() {
        for (const ab of fab) {
            for (const a of fa) {
                yield ab(a);
            }
        }
    },
});
/**
 * @category instances
 * @since 1.0.0
 */
export const Apply = {
    ...Functor,
    ap: (fab, fa) => ap(fa)(fab),
};
/**
 * @category apply
 * @since 1.0.0
 */
export const apFirst = apFirst_(Apply);
/**
 * @category apply
 * @since 1.0.0
 */
export const apSecond = apSecond_(Apply);
/**
 * @category instances
 * @since 1.0.0
 */
export const Applicative = {
    ...Pointed,
    ...Apply,
};
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatMap = (f) => (fa) => ({
    *[Symbol.iterator]() {
        for (const a1 of fa) {
            for (const a2 of f(a1)) {
                yield a2;
            }
        }
    },
});
/**
 * @category sequencing
 * @since 1.0.0
 */
export const flatten = 
/*#__PURE__*/ flatMap(identity);
/**
 * @category instances
 * @since 1.0.0
 */
export const Chain = {
    ...Apply,
    chain: (fab, fa) => flatMap(fa)(fab),
};
/**
 * @category instances
 * @since 1.0.0
 */
export const Unfoldable = {
    URI,
    unfold,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const Monad = {
    ...Pointed,
    ...Chain,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const FromIO = {
    URI,
    fromIO,
};
/**
 * @category instances
 * @since 1.0.0
 */
export const MonadIO = {
    ...Monad,
    fromIO,
};
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export const filterWithIndex = (predicateWithIndex) => (fa) => {
    let i = 0;
    return {
        *[Symbol.iterator]() {
            for (const a of fa) {
                if (predicateWithIndex(i++, a)) {
                    yield a;
                }
            }
        },
    };
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filter = (predicate) => (fa) => ({
    *[Symbol.iterator]() {
        for (const a of fa) {
            if (predicate(a)) {
                yield a;
            }
        }
    },
});
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMapWithIndex = (f) => (fa) => {
    let i = 0;
    return {
        *[Symbol.iterator]() {
            for (const a of fa) {
                const optionB = f(i++, a);
                if (O.isSome(optionB)) {
                    yield optionB.value;
                }
            }
        },
    };
};
/**
 * @category filtering
 * @since 1.0.0
 */
export const filterMap = (f) => filterMapWithIndex((_, a) => f(a));
/**
 * Compacts an Iterable of `Option`s discarding the `None` values and
 * keeping the `Some` values. It returns a new array containing the values of
 * the `Some` options.
 * @category filtering
 * @since 1.0.0
 */
export const compact = 
/*#__PURE__*/ filterMap(identity);
/**
 * @category filtering
 * @since 1.0.0
 */
export const rights = (fa) => ({
    *[Symbol.iterator]() {
        for (const a of fa) {
            if (a._tag === "Right") {
                yield a.right;
            }
        }
    },
});
/**
 * @category filtering
 * @since 1.0.0
 */
export const lefts = (ai) => ({
    *[Symbol.iterator]() {
        for (const a of ai) {
            if (a._tag === "Left") {
                yield a.left;
            }
        }
    },
});
/**
 * Creates a new `Iterable` removing duplicate elements, keeping the first occurrence of an element,
 * based on a `Eq<A>`.
 * @category filtering
 * @since 1.0.0
 */
export const uniq = (E) => (fa) => {
    const uniques = [];
    return {
        *[Symbol.iterator]() {
            for (const a of fa) {
                if (uniques.every((o) => !E.equals(o, a))) {
                    uniques.push(a);
                    yield a;
                }
            }
        },
    };
};
/**
 * @category mapping
 * @since 1.0.0
 */
export function transform(transform, flush) {
    return (fa) => ({
        *[Symbol.iterator]() {
            for (const a of fa) {
                const b = transform(a);
                if (O.isSome(b)) {
                    yield b.value;
                }
            }
            if (flush) {
                yield flush();
            }
        },
    });
}
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export const tap = /*#__PURE__*/ dual(2, tap_(Chain));
/**
 * @category combinators
 * @since 1.0.0
 */
export const tapIO = /*#__PURE__*/ dual(2, tapIO_(FromIO, Chain));
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @category do notation
 * @since 1.0.0
 */
export const Do = /*#__PURE__*/ of({});
/**
 * @category do notation
 * @since 1.0.0
 */
export const bind = chainable.bind(Chain);
/**
 * @category do notation
 * @since 1.0.0
 */
export const bindTo = bindTo_(Functor);
const _let = let_(Functor);
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export const apS = /*#__PURE__*/ apS_(Apply);
/**
 * @category folding
 * @since 1.0.0
 */
export const reduceWithIndex = (b, f) => (fa) => {
    let i = 0;
    let out = b;
    for (const a of fa) {
        out = f(i++, out, a);
    }
    return out;
};
/**
 * @category folding
 * @since 1.0.0
 */
export const reduce = (b, f) => reduceWithIndex(b, (_, b, a) => f(b, a));
/**
 * @category folding
 * @since 1.0.0
 */
export const foldMapWithIndex = (M) => (f) => (fa) => pipe(fa, reduceWithIndex(M.empty, (i, b, a) => M.concat(b, f(i, a))));
/**
 * @category folding
 * @since 1.0.0
 */
export const foldMap = (M) => {
    const foldMapWithIndexM = foldMapWithIndex(M);
    return (f) => foldMapWithIndexM((_, a) => f(a));
};
/**
 * @category conversions
 * @since 1.0.0
 */
export const toArray = (fa) => Array.from(fa);
