/**
 * The AsyncIterable module provides tools for working with AsyncIterable<T> type in a functional way.
 *
 * In functional jargon, this module provides a monadic interface over AsyncIterable<T>.
 *
 * @since 1.0.0
 */
import { Applicative1 } from "fp-ts/lib/Applicative";
import { Apply1 } from "fp-ts/lib/Apply";
import * as chainable from "fp-ts/lib/Chain";
import { Either } from "fp-ts/lib/Either";
import { Eq } from "fp-ts/lib/Eq";
import { PredicateWithIndex, RefinementWithIndex } from "fp-ts/lib/FilterableWithIndex";
import { FromIO1 } from "fp-ts/lib/FromIO";
import { FromTask1 } from "fp-ts/lib/FromTask";
import { Functor1 } from "fp-ts/lib/Functor";
import { FunctorWithIndex1 } from "fp-ts/lib/FunctorWithIndex";
import { IO } from "fp-ts/lib/IO";
import { Monad1 } from "fp-ts/lib/Monad";
import { MonadIO1 } from "fp-ts/lib/MonadIO";
import { Monoid } from "fp-ts/lib/Monoid";
import * as O from "fp-ts/lib/Option";
import { Pointed1 } from "fp-ts/lib/Pointed";
import { Predicate } from "fp-ts/lib/Predicate";
import { Refinement } from "fp-ts/lib/Refinement";
import * as T from "fp-ts/lib/Task";
import { Unfoldable1 } from "fp-ts/lib/Unfoldable";
import { LazyArg } from "fp-ts/lib/function";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export declare const URI = "AsyncIterable";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export type URI = typeof URI;
declare module "fp-ts/lib/HKT" {
    interface URItoKind<A> {
        readonly [URI]: AsyncIterable<A>;
    }
}
/**
 * Return a `AsyncIterable` with elements initialized with `f(i)`.
 *
 * Iterable stops when f return O.none
 *
 * @category constructors
 * @since 1.0.0
 */
export declare const makeByWithIndex: <A>(f: (i: number) => O.Option<A>) => AsyncIterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeBy: <A>(f: () => O.Option<A>) => AsyncIterable<A>;
/**
 * Return a `AsyncIterable` with elements initialized with `f(i)`.
 *
 * Iterable stops when f return O.none
 *
 * @category constructors
 * @since 1.0.0
 */
export declare const makeByTaskWithIndex: <A>(f: (i: number) => T.Task<O.Option<A>>) => AsyncIterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeByTask: <A>(f: () => T.Task<O.Option<A>>) => AsyncIterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const unfold: <A, B>(b: B, f: (b: B) => O.Option<readonly [A, B]>) => AsyncIterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const unfoldTask: <A, B>(b: B, f: (b: B) => T.Task<O.Option<readonly [A, B]>>) => AsyncIterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIterable: <A>(fa: Iterable<A>) => AsyncIterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare function fromAsyncGenerator<A>(fa: LazyArg<AsyncGenerator<A>>): AsyncIterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromLazyArg: <A>(fa: LazyArg<A>) => AsyncIterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIO: <A>(fa: IO<A>) => AsyncIterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromTask: <A>(fa: T.Task<A>) => AsyncIterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const of: Pointed1<URI>["of"];
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Pointed: Pointed1<URI>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const mapWithIndex: <A, B>(f: (index: number, a: A) => B) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Functor: Functor1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FunctorWithIndex: FunctorWithIndex1<URI, number>;
/**
 * Maps every value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const as: {
    <A>(a: A): <_>(self: AsyncIterable<_>) => AsyncIterable<A>;
    <_, A>(self: AsyncIterable<_>, a: A): AsyncIterable<A>;
};
/**
 * Maps every value to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const asUnit: <_>(self: AsyncIterable<_>) => AsyncIterable<void>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: AsyncIterable<(a: A) => B>) => AsyncIterable<B>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const ap: <A>(fa: AsyncIterable<A>) => <B>(fab: AsyncIterable<(a: A) => B>) => AsyncIterable<B>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apTask: <A>(fa: AsyncIterable<A>) => <B>(fab: AsyncIterable<(a: A) => T.Task<B>>) => AsyncIterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Apply: Apply1<URI>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apFirst: <B>(second: AsyncIterable<B>) => <A>(first: AsyncIterable<A>) => AsyncIterable<A>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apSecond: <B>(second: AsyncIterable<B>) => <A>(first: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Applicative: Applicative1<URI>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMap: <A, B>(f: (a: A) => AsyncIterable<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapIterable: <A, B>(f: (a: A) => Iterable<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTaskWithIndex: <A, B>(f: (index: number, a: A) => T.Task<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMapTask: <A, B>(f: (a: A) => T.Task<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatten: <A>(mma: AsyncIterable<AsyncIterable<A>>) => AsyncIterable<A>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flattenIterable: <A>(mma: AsyncIterable<Iterable<A>>) => AsyncIterable<A>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Chain: chainable.Chain1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Unfoldable: Unfoldable1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Monad: Monad1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromIO: FromIO1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromTask: FromTask1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadIO: MonadIO1<URI>;
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export declare const filterWithIndex: {
    <A, B extends A>(refinementWithIndex: RefinementWithIndex<number, A, B>): (fa: AsyncIterable<A>) => AsyncIterable<B>;
    <A>(predicateWithIndex: PredicateWithIndex<number, A>): <B extends A>(fb: AsyncIterable<A>) => AsyncIterable<B>;
    <A>(predicateWithIndex: PredicateWithIndex<number, A>): (fa: AsyncIterable<A>) => AsyncIterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filter: {
    <A, B extends A>(refinement: Refinement<A, B>): (fa: AsyncIterable<A>) => AsyncIterable<B>;
    <A>(predicate: Predicate<A>): <B extends A>(fb: AsyncIterable<B>) => AsyncIterable<B>;
    <A>(predicate: Predicate<A>): (fa: AsyncIterable<A>) => AsyncIterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export interface PredicateTaskWithIndex<I, A> {
    (i: I, a: A): T.Task<boolean>;
}
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTaskWithIndex: {
    <A>(predicateWithIndex: PredicateTaskWithIndex<number, A>): <B extends A>(fb: AsyncIterable<B>) => AsyncIterable<B>;
    <A>(predicateWithIndex: PredicateTaskWithIndex<number, A>): (fa: AsyncIterable<A>) => AsyncIterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export interface PredicateTask<A> {
    (a: A): T.Task<boolean>;
}
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterTask: {
    <A>(predicate: PredicateTask<A>): <B extends A>(fb: AsyncIterable<B>) => AsyncIterable<B>;
    <A>(predicate: PredicateTask<A>): (fa: AsyncIterable<A>) => AsyncIterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapWithIndex: <A, B>(f: (i: number, a: A) => O.Option<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => O.Option<B>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTaskWithIndex: <A, B>(f: (i: number, a: A) => T.Task<O.Option<B>>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapTask: <A, B>(f: (a: A) => T.Task<O.Option<B>>) => (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * Compacts an AsyncIterable of `Option`s discarding the `None` values and
 * keeping the `Some` values. It returns a new array containing the values of
 * the `Some` options.
 * @category filtering
 * @since 1.0.0
 */
export declare const compact: <A>(fa: AsyncIterable<O.Option<A>>) => AsyncIterable<A>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const rights: <E, A>(fa: AsyncIterable<Either<E, A>>) => AsyncIterable<A>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const lefts: <E, A>(fa: AsyncIterable<Either<E, A>>) => AsyncIterable<E>;
/**
 * Creates a new `AsyncIterable` removing duplicate elements, keeping the first occurrence of an element, based on a `Eq<A>`.
 * @category filtering
 * @since 1.0.0
 */
export declare const uniq: <A>(E: Eq<A>) => (fa: AsyncIterable<A>) => AsyncIterable<A>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare function transform<A, B>(transform: (a: A) => O.Option<B>, flush?: () => B): (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare function transformTask<A, B>(transform: (a: A) => T.Task<O.Option<B>>, flush?: () => T.Task<B>): (fa: AsyncIterable<A>) => AsyncIterable<B>;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export declare const tap: {
    <A, _>(self: AsyncIterable<A>, f: (a: A) => AsyncIterable<_>): AsyncIterable<A>;
    <A, _>(f: (a: A) => AsyncIterable<_>): (self: AsyncIterable<A>) => AsyncIterable<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapTask: {
    <A, _>(f: (a: A) => T.Task<_>): (self: AsyncIterable<A>) => AsyncIterable<A>;
    <A, _>(self: AsyncIterable<A>, f: (a: A) => T.Task<_>): AsyncIterable<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapIO: {
    <A, _>(f: (a: A) => IO<_>): (self: AsyncIterable<A>) => AsyncIterable<A>;
    <A, _>(self: AsyncIterable<A>, f: (a: A) => IO<_>): AsyncIterable<A>;
};
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const Do: AsyncIterable<object>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bind: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => AsyncIterable<B>) => (ma: AsyncIterable<A>) => AsyncIterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <A>(fa: AsyncIterable<A>) => AsyncIterable<{ readonly [K in N]: A; }>;
declare const _let: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => B) => (fa: AsyncIterable<A>) => AsyncIterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const apS: <N extends string, A, B>(name: Exclude<N, keyof A>, fb: AsyncIterable<B>) => (fa: AsyncIterable<A>) => AsyncIterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * the order of results does not corresponds to order of async iterator elements
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableLimited<A>(limit: number): (iterable: AsyncIterable<A>) => T.Task<Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterablePar<A>(limit: number): (iterable: AsyncIterable<A>) => T.Task<Iterable<A>>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toIterableSeq<A>(): (iterable: AsyncIterable<A>) => T.Task<Iterable<A>>;
/**
 * preserves the order of elements coming from async iterator and corresponding results
 * @category folding
 * @since 1.0.0
 */
export declare function toArrayLimited<A>(limit: number): (iterable: AsyncIterable<A>) => T.Task<A[]>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArrayPar<A>(limit: number): (iterable: AsyncIterable<A>) => T.Task<A[]>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare function toArraySeq<A>(): (iterable: AsyncIterable<A>) => T.Task<A[]>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const foldMapWithIndexPar: <M>(M: Monoid<M>, limit: number) => <A>(f: (i: number, a: A) => M) => (fa: AsyncIterable<A>) => T.Task<M>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const foldMapPar: <M>(M: Monoid<M>, limit: number) => <A>(f: (a: A) => M) => (fa: AsyncIterable<A>) => T.Task<M>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const foldMapSeq: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: AsyncIterable<A>) => T.Task<M>;
