/**
 * The Iterable module provides tools for working with Typescript's Iterable<T> type in a functional way.
 *
 * In functional jargon, this module provides a monadic interface over Typescript's Iterable<T>.
 *
 * This nodule does not implements fuctions that would lead to degrading perfomance of Iterable. Any Iterable can be easly converted to Array
 *
 * @since 1.0.0
 */
import { Applicative1 } from "fp-ts/lib/Applicative";
import { Apply1 } from "fp-ts/lib/Apply";
import * as chainable from "fp-ts/lib/Chain";
import { Either } from "fp-ts/lib/Either";
import { Eq } from "fp-ts/lib/Eq";
import { PredicateWithIndex, RefinementWithIndex } from "fp-ts/lib/FilterableWithIndex";
import { FromIO1 } from "fp-ts/lib/FromIO";
import { Functor1 } from "fp-ts/lib/Functor";
import { FunctorWithIndex1 } from "fp-ts/lib/FunctorWithIndex";
import { IO } from "fp-ts/lib/IO";
import { Monad1 } from "fp-ts/lib/Monad";
import { MonadIO1 } from "fp-ts/lib/MonadIO";
import { Monoid } from "fp-ts/lib/Monoid";
import { NaturalTransformation11 } from "fp-ts/lib/NaturalTransformation";
import { Option } from "fp-ts/lib/Option";
import { Pointed1 } from "fp-ts/lib/Pointed";
import { Predicate } from "fp-ts/lib/Predicate";
import * as RA from "fp-ts/lib/ReadonlyArray";
import { Refinement } from "fp-ts/lib/Refinement";
import { Unfoldable1 } from "fp-ts/lib/Unfoldable";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export declare const URI = "Iterable";
/**
 * @category type lambdas
 * @since 1.0.0
 */
export type URI = typeof URI;
declare module "fp-ts/lib/HKT" {
    interface URItoKind<A> {
        readonly [URI]: Iterable<A>;
    }
}
/**
 * Return a `Iterable` with elements initialized with `f(i)`.
 * Iterable stops when f return O.none
 *
 * @category constructors
 * @since 1.0.0
 */
export declare const makeByWithIndex: <A>(f: (i: number) => Option<A>) => Iterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeBy: <A>(f: () => Option<A>) => Iterable<A>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const unfold: <A, B>(b: B, f: (b: B) => Option<readonly [A, B]>) => Iterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const fromIO: <A>(fa: IO<A>) => Iterable<A>;
/**
 * @category conversions
 * @since 1.0.0
 * isn't it weird? all arrays are iterables
 */
export declare const fromReadonlyArray: NaturalTransformation11<RA.URI, URI>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const of: Pointed1<URI>["of"];
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Pointed: Pointed1<URI>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const mapWithIndex: <A, B>(f: (index: number, a: A) => B) => (fa: Iterable<A>) => Iterable<B>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: Iterable<A>) => Iterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Functor: Functor1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FunctorWithIndex: FunctorWithIndex1<URI, number>;
/**
 * Maps the value to the specified constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const as: {
    <A>(a: A): <_>(self: Iterable<_>) => Iterable<A>;
    <_, A>(self: Iterable<_>, a: A): Iterable<A>;
};
/**
 * Maps the value to the void constant value.
 *
 * @category mapping
 * @since 1.0.0
 */
export declare const asUnit: <_>(self: Iterable<_>) => Iterable<void>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Iterable<(a: A) => B>) => Iterable<B>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const ap: <A>(fa: Iterable<A>) => <B>(fab: Iterable<(a: A) => B>) => Iterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Apply: Apply1<URI>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apFirst: <B>(second: Iterable<B>) => <A>(first: Iterable<A>) => Iterable<A>;
/**
 * @category apply
 * @since 1.0.0
 */
export declare const apSecond: <B>(second: Iterable<B>) => <A>(first: Iterable<A>) => Iterable<B>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Applicative: Applicative1<URI>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatMap: <A, B>(f: (a: A) => Iterable<B>) => (fa: Iterable<A>) => Iterable<B>;
/**
 * @category sequencing
 * @since 1.0.0
 */
export declare const flatten: <A>(mma: Iterable<Iterable<A>>) => Iterable<A>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Chain: chainable.Chain1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Unfoldable: Unfoldable1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const Monad: Monad1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const FromIO: FromIO1<URI>;
/**
 * @category instances
 * @since 1.0.0
 */
export declare const MonadIO: MonadIO1<URI>;
/**
 * Same as [`filter`](#filter), but passing also the index to the iterating function.
 *
 * @category filtering
 * @since 1.0.0
 */
export declare const filterWithIndex: {
    <A, B extends A>(refinementWithIndex: RefinementWithIndex<number, A, B>): (fa: Iterable<A>) => Iterable<B>;
    <A>(predicateWithIndex: PredicateWithIndex<number, A>): <B extends A>(fb: Iterable<A>) => Iterable<B>;
    <A>(predicateWithIndex: PredicateWithIndex<number, A>): (fa: Iterable<A>) => Iterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filter: {
    <A, B extends A>(refinement: Refinement<A, B>): (fa: Iterable<A>) => Iterable<B>;
    <A>(predicate: Predicate<A>): <B extends A>(fb: Iterable<B>) => Iterable<B>;
    <A>(predicate: Predicate<A>): (fa: Iterable<A>) => Iterable<A>;
};
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMapWithIndex: <A, B>(f: (i: number, a: A) => Option<B>) => (fa: Iterable<A>) => Iterable<B>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => Option<B>) => (fa: Iterable<A>) => Iterable<B>;
/**
 * Compacts an Iterable of `Option`s discarding the `None` values and
 * keeping the `Some` values. It returns a new array containing the values of
 * the `Some` options.
 * @category filtering
 * @since 1.0.0
 */
export declare const compact: <A>(fa: Iterable<Option<A>>) => Iterable<A>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const rights: <E, A>(fa: Iterable<Either<E, A>>) => Iterable<A>;
/**
 * @category filtering
 * @since 1.0.0
 */
export declare const lefts: <E, A>(ai: Iterable<Either<E, A>>) => Iterable<E>;
/**
 * Creates a new `Iterable` removing duplicate elements, keeping the first occurrence of an element,
 * based on a `Eq<A>`.
 * @category filtering
 * @since 1.0.0
 */
export declare const uniq: <A>(E: Eq<A>) => (fa: Iterable<A>) => Iterable<A>;
/**
 * @category mapping
 * @since 1.0.0
 */
export declare function transform<A, B>(transform: (a: A) => Option<B>, flush?: () => B): (fa: Iterable<A>) => Iterable<B>;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * @category combinators
 * @since 1.0.0
 */
export declare const tap: {
    <A, _>(self: Iterable<A>, f: (a: A) => Iterable<_>): Iterable<A>;
    <A, _>(f: (a: A) => Iterable<_>): (self: Iterable<A>) => Iterable<A>;
};
/**
 * @category combinators
 * @since 1.0.0
 */
export declare const tapIO: {
    <A, _>(f: (a: A) => IO<_>): (self: Iterable<A>) => Iterable<A>;
    <A, _>(self: Iterable<A>, f: (a: A) => IO<_>): Iterable<A>;
};
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const Do: Iterable<{}>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bind: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => Iterable<B>) => (ma: Iterable<A>) => Iterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <A>(fa: Iterable<A>) => Iterable<{ readonly [K in N]: A; }>;
declare const _let: <N extends string, A, B>(name: Exclude<N, keyof A>, f: (a: A) => B) => (fa: Iterable<A>) => Iterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
export { 
/**
 * @category do notation
 * @since 1.0.0
 */
_let as let, };
/**
 * @category do notation
 * @since 1.0.0
 */
export declare const apS: <N extends string, A, B>(name: Exclude<N, keyof A>, fb: Iterable<B>) => (fa: Iterable<A>) => Iterable<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B; }>;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const reduceWithIndex: <A, B>(b: B, f: (i: number, b: B, a: A) => B) => (fa: Iterable<A>) => B;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const reduce: <A, B>(b: B, f: (b: B, a: A) => B) => (fa: Iterable<A>) => B;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const foldMapWithIndex: <M>(M: Monoid<M>) => <A>(f: (i: number, a: A) => M) => (fa: Iterable<A>) => M;
/**
 * @category folding
 * @since 1.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: Iterable<A>) => M;
/**
 * @category conversions
 * @since 1.0.0
 */
export declare const toArray: <A>(fa: Iterable<A>) => Array<A>;
